@@ -1,4 +1,8 @@
 # Put this in R/utils-internal.R
+
+# Declare global variables to avoid R CMD check NOTEs for ggplot2 aes() usage
+utils::globalVariables(c("time", "response", "HRF", "Regressor", "onset"))
+
 #' @keywords keyword
 #' @noRd
 recycle_or_error <- function(x, n, name) {
